%% CODE FOR RECESSION ANALYSIS 
%   Written by Tom Mller, PhD student, University of Lausanne, CH
%   (tom.muller.1@unil.ch)
%   Date : 22.02.2022
%   Matlab version : R2019a

clear all
close all
clc

%cd 'D:\Nextcloud\01_PhD\Data_analysis\discharge_analysis1' ;
Acatchment = 30347885;  % Catchment area [m2]

% Load discharge data
load dischargeData2006-17.mat

%% SUMMER MEANS
% NOTE THAT DISCHARGE >10 m3/s was cutoff due to spillage of the water
% intake dam

close all

calcStart = 2007;
calcEnd = 2017;

summerMeans = nan(calcEnd-calcStart+1,2);
k=0

figure
for startYear = (calcStart:calcEnd)
    
    k=k+1;
    yearAnalysis = datetime(startYear,6,1);
    yearAnalysis.TimeZone = '+02:00';
    dateStart = find(QAll.date >= yearAnalysis, 1);
    dateEnd = find(QAll.date >= min(QAll.date(end),datestr(addtodate(datenum(yearAnalysis), 3, 'month'))), 1) -1;
    
    MATY = QAll.discharge(dateStart:dateEnd);
    MATX = QAll.date(dateStart:dateEnd);

    summerMeans(k,:) = [startYear nanmean(MATY)];
    
    subplot(4,3,k)
    plot(MATX,MATY,[MATX(1) MATX(end)],[summerMeans(k,2) summerMeans(k,2)],'r')
    t=text(min(MATX),summerMeans(k,2)+0.5,sprintf('mean = %.2f',summerMeans(k,2)));
    t.Color = 'red';
    title(startYear);
    ylabel('Discharge [m3/s]')
    xlabel('Date')
end

figure
plot(summerMeans(:,1),summerMeans(:,2),'-o')
xlabel('Year')
ylabel('Mean summer discharge [m3/s]')
title('Mean elevation = 3025 m')

figure
plot(summerMeans(:,1),summerMeans(:,2)/Acatchment*1000*24*3600,'-o')
xlabel('Year')
ylabel('Mean summer discharge [mm/d]')
title('Mean elevation = 3025 m')

summerMeans(:,3) = summerMeans(:,2)/Acatchment*1000*24*3600

%% CREATE RECESSION DATA FOR ALL YEARS AND SMOOTHED 
%Recession is taken as individual events and smoothed afterward
close all

timestep = 1;                                   %per hour
avgThreshold = 24*2;                            %USED FOR SMOOTHING of Discharge (2 days) (used to avoid step like discharge decrease)
cutoff = 0.15;                                  %CUT HIGH DISCHARGE DATA IN m3/s (>0.15 m3/s)
recessionThresholdLong = 10*24;                 %MINIMUM RECESSION LENGTH TO IDENTIDY RECESSION in hours (10*24) = 10 days
recessionThreshold = recessionThresholdLong;

recessionThreshold_dQ_dt = 24*5;                %Smoothing (5 days), only used to identify real recession periods

%IMPORTANT MANUAL FACTOR TO CHANGE
smoothFactor = 0.5;                             %fraction of dataset used for smoothing using moving windows (50%)
cutEndLength = 24*5;                            %cuts the last few days of recession (to avoid problems) (5 days)
KircherLinearCut = 0.33;                        %ONLY DO A LINEAR FIT BELOW THIS POINT ! 0.33 mm/day

% cut unrealisitic data points 
cutEndLowQ = 0.11/(3600*24/(Acatchment/1000));  %cuts the recession data which are lower than 0.11 mm/day (only remove a few bad points)
cutEndLowdQ = 2e-3/(3600*24/(Acatchment/1000)); %cuts the dQ/dt data; good value is 2e-3 mm/day^2 (only remove a few bad points)

% SMOOTH DATASET AND REMOVE HIGH VALUES
recessionData = movingMean(QAll.discharge,avgThreshold);
recessionData = recessionData(QAll.date<datetime(2017,5,1,'TimeZone','+02:00'));
recessionData((recessionData)>=cutoff) = nan;

% Calculates dQ/dt
dQ_dt = recessionData(2:end) - recessionData(1:end-1);
dQ_dt = [0;dQ_dt];

% Identify recession periods
decrease = forwardMean(dQ_dt,recessionThreshold_dQ_dt) ;
decrease2 = backwardMean(dQ_dt,recessionThreshold_dQ_dt) ;

decrease(decrease<=0 & decrease2<=0) = 1 ;
decrease(decrease ~= 1) = 0 ;

% Find location of all recessions (decrease=1)
D2 = findFlat2(decrease,recessionThreshold,timestep);

% Create empty dataset for later
recessionData2 = nan(size(D2));
smooth_Qrecess2 = nan(1,length(QAll.discharge));

recessionAutumn = QAll;
recessionAutumn.discharge = nan(length(QAll.date),1);
recessionDate3 = [];
Qrecess2 = nan(1,length(QAll.discharge));

% FOR EVERY YEAR : create a smoothed dataset for each recession (to remove
% step like discharge decrease in order to calculate dQ/dt
for i = 1:length(D2(1,:))
    mem = D2(:,i);
    mem = mem(~isnan(mem));
    
    % STRONG SMOOTHING OF EACH RECESSION (moving window of 50% of dataset)
    temp = smooth(recessionData(mem),smoothFactor,'moving')';
    temp(temp<cutEndLowQ) = nan;
    
    recessionData2(1:length(mem)-cutEndLength,i) = temp(1:length(temp)-cutEndLength);
    smooth_Qrecess2(mem(1:length(mem)-cutEndLength)) = temp(1:length(temp)-cutEndLength);
    Qrecess2(mem) = QAll.discharge(mem);
    recessionAutumn.discharge(mem) = QAll.discharge(mem);
end

% Transform from m3/s to mm/day
smooth_Qrecess = smooth_Qrecess2' / (Acatchment/1000) * 3600 * 24;

% Calculates dQ/dt
dQ_dt = (smooth_Qrecess(2:end) - smooth_Qrecess(1:end-1))/(1/24);
dQ_dt(dQ_dt>0)=nan;
recession_dQ_dt = -dQ_dt;
recession_dQ_dt((recession_dQ_dt) <= cutEndLowdQ) = nan;
Qmean = (smooth_Qrecess(2:end) + smooth_Qrecess(1:end-1))/2;

% PLOT 
figure
ha(1) = subplot(2,1,1)
semilogy(QAll.date,QAll.discharge/(Acatchment/1000) * 3600*24);
hold on
% plot un-smoothed discharge during recession period
semilogy(QAll.date,Qrecess2/(Acatchment/1000) * 3600*24,'.g') %
hold on
grid 'on'
% plot smoothed discharge during recession period
semilogy(QAll.date,smooth_Qrecess,'-')
legend('Discharge','Recessions','SmoothQ')
ylabel('Discharge [mm/day]')

% plot corresponding dQ/dt 
ha(2) = subplot(2,1,2)
semilogy(QAll.date(2:end),(recession_dQ_dt),'.')
grid 'on'
linkaxes(ha, 'x');
ylabel('dQ/dt [mm/day^2]')

%% RECESSION ANALYSIS ON ALL YEARS !
% We will estimate the parameters of the recession relationship S = eQ^c
% The derivative gives -dQ/dt = 1/(ce)Q^(2-c), summarized as -dQ/dt = aQ^b
% with a = 1/ce ; b = 2-c
% We use the method as in "Kirchner J.W., 2009" to perform this analysis 
% In particular we will plot ln(-dQ/dt) vs. ln(Q) to extract the slope coefficient b

Q = Qmean(~isnan(Qmean));
dQdt = recession_dQ_dt(~isnan(Qmean));

paired = [Q,dQdt];

%Bin data
binLim = [log(nanmax(paired(:,1))),log(nanmin(paired(:,1)))];
bins = min(binLim):(max(binLim)-min(binLim))/100:max(binLim);

bindQdt = nan(length(bins)-1,7);
k=1;
for i = 2:length(bins)
    bined = paired((log(paired(:,1))<bins(i))&(log(paired(:,1))>=bins(i-k)),2);

    if (length(bined)<(0.01*(length(paired(:,1)))))
        k = k + 1;
        i;
    else 
        bindQdt(i-1,:) = [i,k,exp((bins(i)+bins(i-k))/2),nanmax(bined),nanmin(bined),nanmean(bined),nanstd(bined)];
        k=1;
    end
    
    if (i ==length(bins))
       bindQdt(i-1,:) = [i,k,exp((bins(i)+bins(i-k))/2),nanmax(bined),nanmin(bined),nanmean(bined),nanstd(bined)];
    end

end

Q = Qmean(~isnan(recession_dQ_dt));
dQdt = recession_dQ_dt(~isnan(recession_dQ_dt));

fo = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[-Inf,-Inf,-Inf],...
               'Upper',[Inf,Inf,Inf],...
               'StartPoint',[-1000,-1000,-1000]);

%fit curve as a 2nd order in log space
ft = fittype('a+b*x+c*x^2','options',fo);

% 1) Fit using every points (2nd order fit in log space)
[curve,gof] = fit(log(Q),log(dQdt),ft);
y1 = exp(curve.c*(log(Q).^2) + (curve.b) *log(Q) + curve.a);

% 2) Fit using bins (2nd order fit in log space)
Q2 = bindQdt(~isnan(bindQdt(:,6)),3);
dQdt2 = bindQdt(~isnan(bindQdt(:,6)),6);

[curve2,gof2] = fit(log(Q2),log(dQdt2),ft);
y2 = exp(curve2.c*(log(Q).^2) + (curve2.b) *log(Q) + curve2.a);

% 3) Fit using bins, but only linear model (1st order fit in log space)
Q3 = bindQdt(~isnan(bindQdt(:,6)),3);
dQdt3 = bindQdt(~isnan(bindQdt(:,6)),6);

dQdt3 = dQdt3(Q3<=KircherLinearCut);    %remove points higher than cutoff (see discussion in article (Mller et al., 2022)
Q3 = Q3(Q3<=KircherLinearCut);          %remove points higher than cutoff (see discussion in article (Mller et al., 2022)

ft = fittype('a+b*x');
[curve4,gof4] = fit(log(Q3),log(dQdt3),ft);
y4 = exp((curve4.b) *log(Q) + curve4.a);

figure
loglog(Q,dQdt,'.')
hold on
grid on
loglog(Q2,dQdt2,'*r')
hold on
loglog(Q,y1,'.g')
hold on
loglog(Q,y2,'.m')
hold on
loglog(Q,y4,'.b')
t=text((min(Q)+(max(Q)-min(Q))*0.05),(min(dQdt)+(max(dQdt)-min(dQdt))*0.99),sprintf('ln(-dQ/dt)=%.2f+%.2f ln(Q)+%.2f ln(Q)^2',curve.a,curve.b,curve.c));%curve2.n));
t.Color = 'g';
t2=text((min(Q)+(max(Q)-min(Q))*0.05),(min(dQdt)+(max(dQdt)-min(dQdt))*0.7),sprintf('ln(-dQ/dt)=%.2f+%.2f ln(Q)+%.2f ln(Q)^2',curve2.a,curve2.b,curve2.c));%curve2.n));
t2.Color = 'm';
t3=text((min(Q)+(max(Q)-min(Q))*0.05),(min(dQdt)+(max(dQdt)-min(dQdt))*0.4),sprintf('ln(-dQ/dt)=%.2f+%.2f ln(Q)',curve4.a,curve4.b));%curve2.n));
t3.Color = 'b';
ylabel('dQ/dt [mm/day^2]')
xlabel('Discharge [mm/day]')

% CLEAN PLOT FOR Mller et al., 2022
figure
loglog((Q),(dQdt),'.','MarkerEdgeColor',[0.650980392156863 0.650980392156863 0.650980392156863])
hold on
grid on
loglog(Q2,dQdt2,'*r')
hold on
loglog(Q,y4,'LineWidth',2,'LineStyle','-','Color',[0 0 1])
hold on

ylabel('-dQ/dt [mm/day^2]')
xlabel('Discharge [mm/day]')
legend({['Winter recession data (2006-2017)'],['100 binned means (equal number of points)'],['Linear regression' 10 sprintf('ln(-dQ/dt)=%.2f+%.2f ln(Q)',curve4.a,curve4.b)]},'Location','northwest')

% save plot for Mller et al., 2022
if 1
    set(gcf, 'Position', [100 100 600 400]);
    set(gcf,'color','w');
    addpath('./export/');
    export_fig dqdt.pdf
end

%% FROM KIRCHNER 2009 : Calculate intergral of the 1/g(Q) to estimate Storage ! (BONUS)
% g(Q) = -dQ/dt / Q
% integral(dS) = intergral(dQ / g(Q)) = integral(1/g(Q)*dQ)
% ln(dQ/dt) = a + b* ln(Q) + c * ln(Q)^2
% => ln(g(Q)) = a + (b-1) * ln(Q) + c * ln(Q)^2
% => 1/g(Q) = 1 / exp(a + (b-1) * ln(Q) + c * ln(Q)^2)

fun = @(x) 1./exp(curve.c*(log(x).^2) + (curve.b-1) *log(x) + curve.a);
fun2 = @(x) 1./exp(curve2.c*(log(x).^2) + (curve2.b-1) *log(x) + curve2.a);

S = [];
S2 = [];
Q2 = min(Q):(max(Q)-min(Q))/1000:max(Q);
for i=Q2
    % using all data
    Fint = integral(fun,min(Q),i);%vpaintegral(y(x),min(Q),i,'MaxFunctionCalls',1e+99);
    % Using only bin classes
    Fint2 = integral(fun2,min(Q),i);%vpaintegral(y(x),min(Q),i,'MaxFunctionCalls',1e+99);

    S = [S, Fint];
    S2 = [S2, Fint2];
end

figure
semilogy(S,Q2,'.g',S2,Q2,'.m')
xlabel('Storage [mm]')
ylabel('Discharge [mm/day]')
grid on

figure
plot(S2,Q2,'.m')
xlabel('Storage [mm]')
ylabel('Discharge [mm/day]')
grid on

fo = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[0,0,0],...
               'Upper',[Inf,Inf,Inf],...
               'StartPoint',[0,0,0]);

ft = fittype('a*x^b-c','options',fo);
[curve,gof] = fit(Q2',S',ft);
yplot = curve.a*Q2.^curve.b-curve.c;
[curve2,gof2] = fit(Q2',S2',ft);
yplot2 = curve2.a*Q2.^curve2.b-curve2.c;

figure
%plot(Q2,yplot,'r')
hold on
plot(Q2,yplot2,'b')
%plot(Q2,S,'.g')
plot(Q2,S2,'.m')
%plot(Q2,yplot,'r')
plot(Q2,yplot2,'b')
ylabel('Storage [mm]')
xlabel('Discharge [mm/day]')
legend(sprintf('S=cQ{^b} : c=%.3f;b=%.3f',curve2.a,curve2.b))%sprintf('S=cQ{^b} : c=%.3f;b=%.3f',curve.a,curve.b),
grid on

QkirchnerMin = min(Q);
QkirchnerMax = max(Q);

figure
subplot(1,2,1)
plot(Q2,1./(fun2(Q2)))
ylabel('Recession constant [day^{-1}]')
xlabel('Discharge [mm/day]')
grid on
subplot(1,2,2)
plot(Q2,(fun2(Q2)))
ylabel('Recession time constant [day]')
xlabel('Discharge [mm/day]')
grid on

recessionAutumnSAVE = recessionAutumn;
recessionTime = 1/(exp(-3.5)*0.33^0.5/2)

%% RECESSION ANALYSIS BY YEARS USING BOUSSINEQ QUADRATIC FITTING
% Theory : S = eQ^c and -dQ/dt = aQ^b with b = 2-c
% here we assume that b = 1.5 (from slope of (b) in the above recession analysis), so c=0.5
% We therefore have the solution : Qt = Q0(1+Q0^0.5/e*t)^-2
% More details in Mller et al., 2022

% We now fit this quadratic curve using the raw discharge data (no smoothing)
% We estimate Q0 and e, based on the quadratic fit 

recessionAutumn = recessionAutumnSAVE;        % Use discharge recession periods (without smoothing)
cutoff = 0.12;                                % Cut discharge higher than 0.12 m3/s

fo = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[0,0],...
               'Upper',[Inf,Inf],...
               'StartPoint',[1,0.001]);

% We now fit a quadratic curve using the raw discharge data (no smoothing) to estimate q and e
ft = fittype('q*(1+q^0.5*(x-300)/e)^(-2)','options',fo);

calcStart = 2006;
calcEnd = 2017; 

alphaRecession = nan(calcEnd-calcStart+1,5);
figure
for startYear = (calcStart:calcEnd)
    
    yearAnalysis = datetime(startYear,6,1);
    yearAnalysis.TimeZone = '+02:00';
    dateStart = find(recessionAutumn.date >= yearAnalysis, 1);
    dateEnd = find(recessionAutumn.date >= min(recessionAutumn.date(end),datestr(addtodate(datenum(yearAnalysis), 1, 'year'))), 1) -1;
    
    MATY = recessionAutumn.discharge(dateStart:dateEnd);
    MATY(MATY>cutoff)=nan;
    
    MATX2 = recessionAutumn.date(dateStart:dateEnd);

    MATX2 = MATX2(~isnan(MATY));
    MATY = MATY(~isnan(MATY));
    MATX = datenum(MATX2) - min(datenum(MATX2));
    MATX = datenum(MATX2) - datenum(startYear,1,1);

    if startYear==2006
        MATY(MATX<329)=nan;
        MATX = MATX(~isnan(MATY));
        MATY = MATY(~isnan(MATY));
    elseif startYear==2009
        MATY(MATX<331)=nan;
        MATX = MATX(~isnan(MATY));
        MATY = MATY(~isnan(MATY));
    elseif startYear==2010
        MATY(:)=nan;
        MATX = MATX(~isnan(MATY));
        MATY = MATY(~isnan(MATY));
    elseif startYear==2015
        MATY(MATX<340)=nan;
        MATX = MATX(~isnan(MATY));
        MATY = MATY(~isnan(MATY));
    elseif startYear==2016
        MATY(MATX<330)=nan;
        MATX = MATX(~isnan(MATY));
        MATY = MATY(~isnan(MATY));
    end
    
    subplot(3,4,(startYear-calcStart)+1);
    plot(MATX,MATY,'.g')
    
    if ~isempty(MATY)
        [curve2,gof2] = fit(MATX,MATY,ft);
        hold 'on'
        plot(curve2,'m')
        legend 'off'
    end
    
    title(startYear)
    ylim([0.02 0.15])
    xlabel('Days of recession')
    ylabel('Discharge [m^3/s]')
    legend(sprintf('Qs=%.3f;c=%.1f*',curve2.q,curve2.e)')
    
    % (alphaRecession(:,1) = year of analysis
    % (alphaRecession(:,2) = e
    % (alphaRecession(:,3) = Q0
    if ~isempty(MATY)
        alphaRecession(startYear-calcStart+1,:) = [startYear,curve2.e,curve2.q,gof2.rmse,gof2.rsquare];
    else
        alphaRecession(startYear-calcStart+1,:) = [startYear nan nan nan nan];
    end
end

% (alphaRecession(:,2) = e
% (alphaRecession(:,3) = Q0

alpha = 1./(alphaRecession(:,3).^0.5./alphaRecession(:,2))'
Q0 = alphaRecession(:,3)'

%% RECESSION ANALYSIS BY YEARS USING BOUSSINEQ QUADRATIC FITTING (same as before but different fit)
% Theory : S = eQ^c and -dQ/dt = aQ^b with b = 2-c
% here we assume that b = 1.5 (from slope of (b) in the above recession analysis), so c=0.5
% We therefore have the solution : Qt = Q0(1+Q0^0.5/e*t)^-2
% More details in Mller et al., 2022

% We now fit this quadratic curve using the raw discharge data (no smoothing)
% We estimate Q0 and e, based on the quadratic fit 

recessionAutumn = recessionAutumnSAVE;        % Use discharge recession periods (without smoothing)
cutoff = 0.12;                                % Cut discharge higher than 0.12 m3/s

fo = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[0,0],...
               'Upper',[Inf,Inf],...
               'StartPoint',[1,0.001]);

% We now fit a quadratic curve using the raw discharge data (no smoothing) to estimate q and e
ft = fittype('q*(1+q^0.5*(x)/e)^(-2)','options',fo);

calcStart = 2006;
calcEnd = 2017;

alphaRecession2 = nan(calcEnd-calcStart+1,5);
figure
for startYear = (calcStart:calcEnd)
    
    yearAnalysis = datetime(startYear,6,1);
    yearAnalysis.TimeZone = '+02:00';
    dateStart = find(recessionAutumn.date >= yearAnalysis, 1);
    dateEnd = find(recessionAutumn.date >= min(recessionAutumn.date(end),datestr(addtodate(datenum(yearAnalysis), 1, 'year'))), 1) -1;
    
    MATY = recessionAutumn.discharge(dateStart:dateEnd);
    MATY(MATY>cutoff)=nan;
    
    MATX2 = recessionAutumn.date(dateStart:dateEnd);
    MATX2 = MATX2(~isnan(MATY));
    MATY = MATY(~isnan(MATY))*24*3600/(Acatchment/1000);
   
    if startYear==2006
        MATY(MATX2<datetime(2006,11,22,'Timezone','+02:00'))=nan;
        MATX2 = MATX2(~isnan(MATY));
        MATY = MATY(~isnan(MATY));%*24*3600;
    elseif startYear==2009
        MATY(MATX2<datetime(2009,11,24,'Timezone','+02:00'))=nan;
        MATX2 = MATX2(~isnan(MATY));
        MATY = MATY(~isnan(MATY));%*24*3600;
    elseif startYear==2010
        MATY(:)=nan;
        MATX2 = MATX2(~isnan(MATY));
        MATY = MATY(~isnan(MATY));%*24*3600;
    elseif startYear==2015
        MATY(MATX2<datetime(2015,12,08,'Timezone','+02:00'))=nan;
        MATX2 = MATX2(~isnan(MATY));
        MATY = MATY(~isnan(MATY));%*24*3600;
    elseif startYear==2016
        MATY(MATX2<datetime(2016,11,23,'Timezone','+02:00'))=nan;
        MATX2 = MATX2(~isnan(MATY));
        MATY = MATY(~isnan(MATY));%*24*3600;
    end
    
    MATX = datenum(MATX2) - min(datenum(MATX2));

    subplot(3,4,startYear-calcStart+1);
    plot(MATX,MATY,'.g')
    if ~isempty(MATY)
        [curve2,gof2] = fit(MATX,MATY,ft);
        hold 'on'
        plot(curve2,'m')
        legend 'off'
    end
    title(startYear)
    xlabel('Days of recession')
    ylabel('Discharge [m^3/day]')

    % (alphaRecession2(:,1) = year of analysis
    % (alphaRecession2(:,2) = e
    % (alphaRecession2(:,3) = Q0
    if ~isempty(MATY)
        alphaRecession2(startYear-calcStart+1,:) = [startYear,curve2.e,curve2.q,gof2.rmse,gof2.rsquare];
    else
        alphaRecession2(startYear-calcStart+1,:) = [startYear nan nan nan nan];
    end
end

% (alphaRecession2(:,2) = e
% (alphaRecession2(:,3) = Q0

alpha = 1./(alphaRecession2(:,3).^0.5./alphaRecession2(:,2))'
Q0 = alphaRecession2(:,3)'./24/3600

%% Calculate baseflow storage 
% Now we have estimated e and Q0, which gives us alpha = Q0^0.5/e
% We can finally calculate S_t = e*Q_t^0.5 and S_max = e*Q_max^0.5, where 
% Q_max is the baseflow discharge at the start of the recession.

close all;
recessionAutumn = QAll;

calcStart = 2006;
calcEnd = 2017;

threshLimit = 0.01;

Qrecess = nan(1,startYear-calcStart+1);
for startYear = (calcStart:calcEnd)
    
    yearAnalysis = datetime(startYear,8,1);
    yearAnalysis.TimeZone = '+02:00';
    dateStart = find(recessionAutumn.date >= yearAnalysis, 1);
    dateEnd = find(recessionAutumn.date >= min(recessionAutumn.date(end),datestr(addtodate(datenum(yearAnalysis), 9, 'month'))), 1) -1;
    
    MATY = recessionAutumn.discharge(dateStart:dateEnd);
    MATX2 = recessionAutumn.date(dateStart:dateEnd);
   
    MATX = datenum(MATX2) - datenum(startYear,1,1);
    
    MATXnan = MATX(~isnan(MATY));
    MATYnan = MATY(~isnan(MATY));
    
    % Now we locate the start of the recession and Q_max (that is when the
    % quadratic fit is very close to the real discharge in the early
    % recession (difference is less than threshLimit = 0.01 m3/s)
     
    q = alphaRecession(alphaRecession(:,1)==startYear,3);
    e = alphaRecession(alphaRecession(:,1)==startYear,2);
    error = MATYnan - (q*(1+q^0.5*(MATXnan-300)/e).^-2);
    error = movingMean(abs(error),24);
    
    Qposition = nan;
    if ~isempty(find(error<10^-1,1))
        a = error(1:find(error<(10^-3),1));
        b = find(a > threshLimit);

        Qposition = MATXnan(b(end)+1);
        Qerror = error(b(end));
        Qrecess(startYear-calcStart+1) = (q*(1+q^0.5*(Qposition-300)/e)^-2);   %=Q_max
    end

    %transformation from m3/s to mm/day
    mmtrans = (Acatchment/1000) / 24 / 3600;
    
    % plot recession results in mm/day
    figure(2)
    subplot(3,4,startYear-calcStart+1);
    plot(MATX,log10(MATY/mmtrans),'b',MATX,log10((q*(1+q^0.5*(MATX-300)/e).^-2)/mmtrans),'r',Qposition,log10(Qrecess(startYear-calcStart+1)/mmtrans),'*g','MarkerSize',10)
    title(startYear)  
    ylim([-1. 0.5])
    xlim([250 500])
    grid 'on'
    xlabel('Day of year')
    ylabel('log10 Discharge [mm/day]')

    q2 = alphaRecession2(alphaRecession2(:,1)==startYear,3);
    e2 = alphaRecession2(alphaRecession2(:,1)==startYear,2);
    alpha2 = (q2.^0.5./e2);
    if isnan(alpha2)
        legend('Discharge')
    else   
        legend('Discharge',sprintf('e=%.2f,c=0.5',e2),sprintf('Q_{0}=%.2fmm/day',(Qrecess(startYear-calcStart+1)/mmtrans)))
    end
    
    if 0
        % Same plot in m3/s
        figure(3)
        subplot(3,4,startYear-calcStart+1);
        plot(MATX,log10(MATY),'b',MATX,log10((q*(1+q^0.5*(MATX-300)/e).^-2)),'r',Qposition,log10(Qrecess(startYear-calcStart+1)),'*g','MarkerSize',10)
        title(startYear)  
        ylim([-2. 0.5])
        xlim([250 500])
        grid 'on'
        xlabel('Day of year')
        ylabel('log10 Discharge [m^3/s]')

        if isnan(alpha2)
            legend('Discharge')
        else   
            legend('Discharge',sprintf('e=%.2f,c=0.5',e2),sprintf('Q_{0}=%.2fm^3/s',(Qrecess(startYear-calcStart+1))))
        end
    end
end

% Save plot for Mller et al., 2022
if 1
    set(gcf, 'Position', [100 100 1300 700]);
    set(gcf,'color','w');
    addpath('./export/');
    export_fig recession_years.pdf
end

% Transform Q_max from m3/s to mm/day
Qrecess2 = Qrecess/mmtrans;

% S_max CALCULATED FROM QUADRATIC FIT in mm
S = Qrecess2.^0.5.*alphaRecession2(:,2)';             

% BONUS:
if 1
    % Minimum storage for QUADRATIC FIT at the end of recession using a single low discharge value
    Smin = (QkirchnerMin).^0.5.*alphaRecession2(:,2)';

    % NOW FOR KIRCHNER, Intergration between Qmin and Qmax for each year
    SkirchnerMax = [];
    for i = Qrecess2
        %i = 0.4*(Acatchment/1000) % TO BYPASS YEARLY FLUCTUATIONS 
        SkirchnerMax = [SkirchnerMax,integral(fun2,QkirchnerMin,i)];
    end

    % Percentage diff between Kirchner and direct quadratic boussineq
    (1 - (S-Smin) ./ (SkirchnerMax))*100
    
    figure
    plot(calcStart:calcEnd,(S-Smin),'x-r')
    hold on
    plot(calcStart:calcEnd,SkirchnerMax,'x-b')
    plot(calcStart:calcEnd,S,'x-g')
    plot(calcStart:calcEnd,Smin,'x-m')
    title('Baseflow storage')
    ylabel('S [mm]')
    ylim([0 50])
    %yticks([0 2e5 4e5 6e5 8e5 1e6 1.2e6])
    xlabel('Year of analysis')
    xlim([2005 2018])
    legend('Boussineq Smax-Smin','Kirchner Smax-Smin','Boussineq Smax','Boussineq Smin')
    grid 'on'
end
%%
% FINAL PLOT
figure
subplot(1,3,1)
plot(calcStart:calcEnd,Qrecess2,'.-b','MarkerSize',16)
ylabel('Max. baseflow discharge [mm d^{-1}]')
legend('Q_0')
ylim([0 0.6])
xlim([2005 2018])
grid 'on'

subplot(1,3,2)
plot(calcStart:calcEnd,1./((alphaRecession2(:,3).^0.5)./alphaRecession2(:,2)),'.-g','MarkerSize',16)
legend('1/\alpha')
ylabel('Recession time constant [days]')
ylim([0 180])
xlim([2005 2018])
grid 'on'

subplot(1,3,3)
plot(calcStart:calcEnd,(S),'.-r','MarkerSize',16)
legend('S_{0}')
ylabel('Max. baseflow storage [mm]')
ylim([0 80])
xlim([2005 2018])
grid 'on'

% Save plot for Mller et al., 2022
if 1
    set(gcf, 'Position', [200 200 1200 250]);
    set(gcf,'color','w');
    addpath('./export/');
    export_fig recession_parameters.pdf
end

%% FUNCTIONS

function averageVector = movingMean(x,n)
    averageVector = zeros(length(x),1);
    for i = 1:length(x)
        a = x(max(i-n,1):min(i+n,length(x)));
        averageVector(i) = nanmean(a);
        if (isnan(x(i)))
            averageVector(i) = nan;
        end
    end
end

function averageVector = backwardMean(x,n)
    averageVector = zeros(length(x),1);
    for i = 1:length(x)
        a = x(max(i-n,1):min(i,length(x)));
        averageVector(i) = nanmean(a);
        if (isnan(x(i)))
            averageVector(i) = nan;
        end
    end
end

function averageVector = forwardMean(x,n)
    averageVector = zeros(length(x),1);
    for i = 1:length(x)
        a = x(max(i,1):min(i+n,length(x)));
        averageVector(i) = nanmean(a);
        if (isnan(x(i)))
            averageVector(i) = nan;
        end
    end
end

function flatPosition = findFlat2(x,threshold,timestep)

    cs = cumsum(x);
    x = cs - cummax((x == 0) .* cs);

    C = find( (x(1:end-1) - x(2:end)) > threshold);
    D = nan(24*31*6*timestep);
    
    k=0;
    for i = 1:length(C)
        k=k+1;
        seq1 = (C(i)-x(C(i))+1):1:C(i) ;
        D(1:length(seq1),i) = seq1;
    end

    sum(C>0)
    flatPosition = D(1:find(nansum(D') == 0,1)-1,1:find(isnan(D(1,:) ),1)-1);
end

